@@ -10,7 +10,7 @@
 
 #include <stdio.h>
 #include <setjmp.h>
-#if defined (msdos) || defined (__os2_16__)
+#if defined (__MSDOS__) || defined (__os2_16__)
 # include <process.h>
 # include <dos.h>
 #endif
@@ -415,7 +415,7 @@ static Line *create_line_from_bunch(void) /*{{{*/
    
    /* failed so now malloc new bunch */
    
-   if (NULL == (b = (Bunch_Lines_Type *) jed_malloc (sizeof(Bunch_Lines_Type))))
+   if (NULL == (b = (Bunch_Lines_Type *) SLmalloc (sizeof(Bunch_Lines_Type))))
      return NULL;
    
    if (Bunch_Lines == NULL)
@@ -460,7 +460,7 @@ static void destroy_bunch_line(Line *l) /*{{{*/
      {
 	ll = b->lines;
 	if (
-#if (defined (msdos) && !defined(__WATCOMC__) && !defined(__WIN32__)) || defined (__os2_16__)
+#if defined(SIXTEEN_BIT_SYSTEM)
 	    /* stupid DOS and its memory segmentation forces me to consider
 	     segment then offset */
 	    (FP_SEG(ll) == FP_SEG(l)) &&
@@ -500,7 +500,7 @@ static void destroy_bunch_line(Line *l) /*{{{*/
 		       Next_Free_Offset = BUNCH_SIZE;
 		    }
 		  
-		  SLFREE(b);
+		  SLfree ((char *)b);
 		  
 		  if (last == b) last = NULL;
 		  b = NULL;
@@ -533,7 +533,7 @@ Line *make_line1(int size) /*{{{*/
    unsigned int chunk;
    
    /* 4 byte chunks */
-#if defined(msdos) && !defined(__WIN32__)
+#if defined(SIXTEEN_BIT_SYSTEM)
    chunk = (unsigned) (size + 3) & 0xFFFCU;
 #else
    chunk = ((unsigned) (size + 3)) & 0xFFFFFFFCU;
@@ -546,7 +546,7 @@ Line *make_line1(int size) /*{{{*/
 	     data = NewLine_Buffer;
 	     chunk = 1;
 	  }
-	else data = (unsigned char *) jed_malloc (chunk);   /* was chunk + 1 */
+	else data = (unsigned char *) SLmalloc (chunk);   /* was chunk + 1 */
      }
    
    if ((new_line == NULL) || (data == NULL))
@@ -618,7 +618,7 @@ void free_line(Line *line) /*{{{*/
 {
    register unsigned char *dat = line->data;
    
-   if (dat != NewLine_Buffer) SLFREE(dat);
+   if (dat != NewLine_Buffer) SLfree ((char *)dat);
    destroy_bunch_line(line);
 }
 
@@ -662,7 +662,7 @@ unsigned char *remake_line(int size) /*{{{*/
 {
    unsigned char *d = CLine->data;
    unsigned int mask;
-#if defined(msdos) && !defined(__WIN32__)
+#if defined(SIXTEEN_BIT_SYSTEM)
    mask = 0xFFFCu;
 #else
    mask = 0xFFFFFFFCu;
@@ -671,7 +671,7 @@ unsigned char *remake_line(int size) /*{{{*/
    
    if (d == NewLine_Buffer)
      {
-	if (NULL != (d = (unsigned char *) jed_malloc (size))) *d = '\n';
+	if (NULL != (d = (unsigned char *) SLmalloc (size))) *d = '\n';
      }
    else
      {
@@ -680,7 +680,7 @@ unsigned char *remake_line(int size) /*{{{*/
 	if (((CLine->len + 3) & mask) == size) return d;
 #endif
 #endif
-	d = (unsigned char *) jed_realloc ((char *) d, size);
+	d = (unsigned char *) SLrealloc ((char *) d, size);
      }
    
    if (d == NULL)
@@ -700,49 +700,56 @@ unsigned char *remake_line(int size) /*{{{*/
 
 /*}}}*/
 
-void uniquely_name_buffer(char *trry) /*{{{*/
+void uniquely_name_buffer (Buffer *b, char *trry) /*{{{*/
 {
-   Buffer *b;
-   int exists, version = 0, n;
-   char neew[48];
+   Buffer *bnext;
+   int version = 0, n;
+   char neew[50];
+   char *name;
    
-   *CBuf->name = 0;
    strncpy(neew, trry, 45);  neew[45] = 0;
    n = strlen(neew);
    while (1)
      {
-	exists = 0;
-	b = CBuf->next;
-	while(b != CBuf)
+	bnext = b->next;
+	while (bnext != b)
 	  {
-	     if (!strcmp(neew, b->name)) exists = 1;
-	     b = b->next;
+	     if ((bnext->name != NULL)
+		 && (0 == strcmp(neew, bnext->name)))
+	       break;
+
+	     bnext = bnext->next;
 	  }
-	if (!exists)
+	
+	if (bnext == b)
 	  {
-	     strcpy(CBuf->name,neew);
-	     return;
+	     name = SLang_create_slstring (neew);
+	     if (name == NULL)
+	       exit_error ("Out of memory", 0);
+	     SLang_free_slstring (b->name);
+	     b->name = name;
+	     break;
 	  }
+
 	version++;
-	sprintf(neew + n, "<%d>", version);
+	sprintf (neew + n, "<%d>", version);
      }
 }
 
 /*}}}*/
 
 /* make a buffer and insert it in the list */
-Buffer *make_buffer (void) /*{{{*/
+Buffer *make_buffer (char *name, char *dir, char *file) /*{{{*/
 {
    Buffer *newB;
    
-   newB = (Buffer *) jed_malloc (sizeof(Buffer));
+   newB = (Buffer *) SLmalloc (sizeof(Buffer));
+   
    if (newB == NULL)
-     {
-	exit_error("make_buffer: malloc error", 0);
-     }
-   
+     exit_error("Out of memory", 0);
+
    SLMEMSET ((char *) newB, 0, sizeof(Buffer));
-   
+
    newB->keymap = Global_Map;
    newB->c_time = sys_time();
    newB->local_vars.tab = Jed_Tab_Default;
@@ -763,11 +770,20 @@ Buffer *make_buffer (void) /*{{{*/
 	CBuf->prev = newB;
      }
    
-#ifdef pc_system
+#ifdef IBMPC_SYSTEM
    newB->flags |= ADD_CR_ON_WRITE_FLAG;
 #endif
    newB->syntax_table = Default_Syntax_Table;
-   return(newB);
+
+   buffer_filename (newB, dir, file);
+   if ((name != NULL)
+       || (newB->name == NULL))
+     {
+	if (name == NULL) name = "*scratch*";
+	uniquely_name_buffer (newB, name);
+     }
+
+   return (newB);
 }
 
 /*}}}*/
@@ -786,14 +802,14 @@ void delete_buffer(Buffer *buf) /*{{{*/
      {
 	n = l -> next;
 	free_line(l);
-	/* SLFREE( l->data); SLFREE( l); */
+	/* SLfree( l->data); SLfree( l); */
      }
 
    m = buf->mark_array;
    while (m != NULL)
      {
 	m1 = m->next;
-	SLFREE (m);
+	SLfree ((char *)m);
 	m = m1;
      }
    
@@ -801,10 +817,10 @@ void delete_buffer(Buffer *buf) /*{{{*/
    while (m != NULL)
      {
 	m1 = m->next;
-	SLFREE (m);
+	SLfree ((char *)m);
 	m = m1;
      }
-   
+
    if (buf->user_marks != NULL) free_user_marks (buf);
    
    if (buf->undo != NULL) delete_undo_ring (buf);
@@ -819,7 +835,7 @@ void delete_buffer(Buffer *buf) /*{{{*/
 #endif
    
 #if JED_HAS_COLOR_COLUMNS
-   if (buf->column_colors != NULL) SLFREE (buf->column_colors);
+   SLfree ((char *)buf->column_colors);
 #endif
    buf->prev->next = buf->next;
    buf->next->prev = buf->prev;
@@ -828,7 +844,11 @@ void delete_buffer(Buffer *buf) /*{{{*/
    if (buf->subprocess) jed_kill_process (buf->subprocess - 1);
 #endif
    
-   SLFREE( buf);
+   SLang_free_slstring (buf->dir);
+   SLang_free_slstring (buf->file);
+   SLang_free_slstring (buf->name);
+
+   SLfree ((char *)buf);
 }
 
 /*}}}*/
@@ -884,11 +904,11 @@ Line *dup_line(Line *l) /*{{{*/
    n = l->len;
 #endif
    
-   neew = (Line *) jed_malloc (sizeof(Line));
+   neew = (Line *) SLmalloc (sizeof(Line));
    if ((neew == NULL) ||
-       (NULL == (neew->data = (unsigned char *) jed_malloc (n))))
+       (NULL == (neew->data = (unsigned char *) SLmalloc (n))))
      {
-	exit_error("Malloc Error in dup_line.", 0);
+	exit_error("Out of memory", 0);
      }
    neew->next = l->next;
    neew->prev = l->prev;
@@ -917,11 +937,11 @@ Buffer *find_buffer(char *name) /*{{{*/
      }
    while(b != CBuf);
    
-#ifdef __os2__
+#if !JED_FILE_PRESERVE_CASE
    b = CBuf;
    do
      {
-	if (!strcmpi(b->name, name)) return(b);
+	if (!jed_case_strcmp (b->name, name)) return(b);
 	b = b->next;
      }
    while(b != CBuf);
@@ -986,6 +1006,7 @@ int switch_to_buffer(Buffer *buf) /*{{{*/
      }
    CBuf->line = CLine;
    CBuf->point = Point;
+   jed_set_umask (CBuf->umask);
    return(1);
 }
 
@@ -1028,41 +1049,56 @@ Buffer *find_file_buffer(char *file) /*{{{*/
 #else
 	m = strlen(b->dir);
 	
-# ifndef __os2__
-	if ((m == n) && (!strcmp(b->file, f))
-	    && (!strncmp(b->dir, file, (int) n)))
+	if ((m == n) 
+# if JED_FILE_PRESERVE_CASE
+	    && (0 == strcmp(b->file, f))
+	    && (0 == strncmp(b->dir, file, (unsigned int) n))
 # else
-	  if ((m == n) && (!strcmpi(b->file,f))
-	      && (!strncmpi(b->dir, file, (int) n)))
+	    && (0 == jed_case_strcmp (b->file,f))
+	    && (0 == jed_case_strncmp (b->dir, file, (unsigned int) n))
 # endif
-	    return(b);
+	    )
+	  return b;
 #endif
 	b = b->next;
      }
    while (b != CBuf);
    
-   return(NULL);
+   return NULL;
 }
 
 /*}}}*/
 
 /* take a dir and a filename, expand them then put in buffer structure */
-void buffer_filename(char *dir, char *file) /*{{{*/
+void buffer_filename(Buffer *b, char *dir, char *file) /*{{{*/
 {
-   strcpy (CBuf->dir, dir);
+   unsigned int dirlen;
    
-   if ((file == NULL) || (*file == 0))
+   if (dir == NULL)
+     dir = get_cwd ();
+
+   if (file == NULL)
      {
-	file = extract_file(CBuf->dir);
-	strcpy(CBuf->file, file);
-	*file = 0;
+	file = extract_file (dir);
+	dirlen = (unsigned int) (file - dir);
      }
-   else strcpy(CBuf->file, file);
+   else
+     dirlen = strlen (dir);
    
+   SLang_free_slstring (b->file);
+   SLang_free_slstring (b->dir);
+   
+   if ((NULL == (b->file = SLang_create_slstring (file)))
+       || (NULL == (b->dir = SLang_create_nslstring (dir, dirlen))))
+     {
+	exit_error ("Out of memory", 0);
+     }
+
 #ifdef REAL_UNIX_SYSTEM
-   get_inode_info (CBuf->dir, &CBuf->device, &CBuf->inode);
+   get_inode_info (b->dir, &b->device, &b->inode);
 #endif
-   uniquely_name_buffer(CBuf->file);
+
+   if (*file) uniquely_name_buffer (b, file);
 }
 
 /*}}}*/
@@ -1125,7 +1161,7 @@ void check_line() /*{{{*/
 {
    if ((CLine->len == 1) && (*CLine->data == '\n') && (CLine->data != NewLine_Buffer))
      {
-	SLFREE(CLine->data);
+	SLfree ((char *)CLine->data);
 	CLine->data = NewLine_Buffer;
 #ifdef KEEP_SPACE_INFO
 	CLine->space = 1;
