@@ -1,10 +1,13 @@
+/* -*- mode: C; mode: fold; -*- */
 /*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
+ *  Copyright (c) 1992, 1996 John E. Davis  (davis@space.mit.edu)
  *  All Rights Reserved.
  */
+#include "config.h"
+#include "jed-feat.h"
+
 #include <stdio.h>
 #include <string.h>
-#include "config.h"
 #include "buffer.h"
 #include "ins.h"
 #include "misc.h"
@@ -14,7 +17,7 @@
 
 /* breaks line at current point leaving point at end of current line */
 
-int split_line (void)
+int split_line (void) /*{{{*/
 {
     int size;
    
@@ -29,7 +32,7 @@ int split_line (void)
 	exit_error("Allocation Failure in split_line", 0);
      }
 
-   MEMCPY((char *) CLine->data, (char *) (CLine->prev->data + Point), size);
+   SLMEMCPY((char *) CLine->data, (char *) (CLine->prev->data + Point), size);
    CLine->len = size;
    CLine = CLine->prev;  LineNum--;
    CLine->len = Point;
@@ -40,7 +43,9 @@ int split_line (void)
    return(0);
 }
 
-void splice_line (void)
+/*}}}*/
+
+void splice_line (void) /*{{{*/
 {
    int n1, n2;
 
@@ -57,7 +62,7 @@ void splice_line (void)
 #endif
      remake_line(n1 + n2 + 1);
    
-   MEMCPY((char *) (CLine->data + Point), (char *) CLine->next->data, n2);
+   SLMEMCPY((char *) (CLine->data + Point), (char *) CLine->next->data, n2);
    CLine->len = n1 + n2;
 
    CLine = CLine->next;  LineNum++;
@@ -66,3 +71,5 @@ void splice_line (void)
    pop_spot();
 }
 
+/*}}}*/
+
