@@ -8,12 +8,18 @@
 
 extern void exit_error(char *, int);
 extern void msg_error(char *);
+extern void jed_verror (char *, ...);
 extern void message(char *);
+extern void jed_vmessage (int, char *, ...);
+extern void clear_message (void);
 extern void read_string(char *, int *);
 extern void do_error(void);
 extern void clear_error(void);
 extern int jed_getkey(void);
 
+extern char *jed_malloc (unsigned int);
+extern char *jed_realloc (char *, unsigned int);
+
 extern int begin_keyboard_macro(void);
 extern int macro_query(void);
 extern int end_keyboard_macro(void);
@@ -40,6 +46,13 @@ extern int get_macro(void);
 extern int Exit_From_MiniBuffer;
 extern char *Macro_Buffer_Ptr;
 extern char *read_from_minibuffer(char *, char *, char *, int *);
+extern char *safe_strcat (char *, char *, unsigned int);
+extern char *safe_strcpy (char *, char *, unsigned int);
+
+#if !JED_FILE_PRESERVE_CASE
+extern int jed_case_strncmp (char *, char *, unsigned int);
+extern int jed_case_strcmp (char *, char *);
+#endif
 
 #define IS_MINIBUFFER (JWindow->top == *tt_Screen_Rows)
 
