@@ -1,12 +1,16 @@
+/* -*- mode: C; mode: fold; -*- */
 /*
  *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
  *  All Rights Reserved.
  */
     
+#include "config.h"
+#include "jed-feat.h"
+/*{{{ Include Files */
+
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 #include "buffer.h"
 #include "ins.h"
 #include "line.h"
@@ -15,19 +19,19 @@
 #include "misc.h"
 #include "cmds.h"
 
-/* SLang user object be larger than 100 */
-#define JED_MARK_TYPE 100
+/*}}}*/
+
+/* SLang user object be larger than 128 */
+#define JED_MARK_TYPE 128
 
 Buffer *Paste_Buffer;
 Buffer *Rectangle_Buffer;
 
-
-
 /* This is used by the narrow command so that multiple windows are 
  * handled properly. The assumption is that we are dealing with a canonical
  * region */
       
-static void touch_windows(void)
+static void touch_windows(void) /*{{{*/
 {
    Window_Type *w;
    Line *l;
@@ -56,9 +60,10 @@ static void touch_windows(void)
    pop_mark(&Number_Zero);
 }
 
+/*}}}*/
 
 #if 0
-static int line_in_buffer(Line *line)
+static int line_in_buffer(Line *line) /*{{{*/
 {
    Line *l;
    
@@ -66,11 +71,40 @@ static int line_in_buffer(Line *line)
    while (l != NULL) if (l == line) return(1); else l = l->next;
    return(0);
 }
+
+/*}}}*/
 #endif
 
+/*{{{ Spot/Mark Functions */
 
+static void init_mark (Mark *m, unsigned int flags) /*{{{*/
+{
+   m->line = CLine;
+   m->point = Point;
+   m->n = LineNum + CBuf->nup;
+   m->flags = flags;
+}
+
+/*}}}*/
+
+#if JED_HAS_SAVE_NARROW
+static Mark *create_mark (unsigned int flags) /*{{{*/
+{
+   Mark *m;
+   
+   if (NULL == (m = (Mark *) SLmalloc (sizeof(Mark))))
+      {
+	  exit_error("create-mark: malloc error", 0);
+      }
+   init_mark (m, flags);
+   
+   return m;
+}
+
+/*}}}*/
+#endif   
 /* with prefix argument, pop marks */
-int set_mark_cmd()
+int set_mark_cmd (void) /*{{{*/
 {
    Mark *m;
    if (Repeat_Factor != NULL)
@@ -82,12 +116,13 @@ int set_mark_cmd()
    
    if (CBuf->marks == NULL)
      {
-	push_mark();
+	push_mark ();
      }
+
    m = CBuf->marks;
-   m->line = CLine;
-   m->point = Point;
-   m->n = LineNum + CBuf->nup;
+   
+   init_mark (m, m->flags);
+   
    if ((m->flags & VISIBLE_MARK) == 0) 
      {
 	m->flags |= VISIBLE_MARK;
@@ -96,62 +131,84 @@ int set_mark_cmd()
    
    /* if (m != CBuf->marks) m->next = CBuf->marks;
     CBuf->marks = m; */
-   if (Last_Key_Function == (VOID *) set_mark_cmd) message("Mark Set.");
+   if (Last_Key_Function == (FVOID_STAR) set_mark_cmd) message("Mark Set.");
    return(1);
 }
 
-int push_spot()
+/*}}}*/
+
+static Mark *allocate_mark (Jed_Mark_Array_Type **ap, Mark **mp, unsigned int flags) /*{{{*/
 {
-    Mark *m;
-
-   if (CBuf->spot_ptr < SPOT_ARRAY_SIZE)
+   Jed_Mark_Array_Type *a, *b;
+   Mark *m;
+   
+   a = *ap;
+   if ((a == NULL)
+       || (a->num_marks == JED_MAX_MARK_ARRAY_SIZE))
      {
-	m = &CBuf->spot_array[CBuf->spot_ptr++];
+	if (NULL == (b = (Jed_Mark_Array_Type *) SLmalloc (sizeof(Jed_Mark_Array_Type))))
+	  {
+	     exit_error("allocate_mark: malloc error", 0);
+	  }
+	memset ((char *) b, 0, sizeof (Jed_Mark_Array_Type));
+	b->next = a;
+	a = b;
+	*ap = a;
      }
-   else if (NULL == (m = (Mark *) SLMALLOC(sizeof(Mark))))
-      {
-	  exit_error("push_spot: malloc error!", 0);
-      }
-
-   m->line = CLine;
-   m->point = Point;
-   m->n = LineNum + CBuf->nup;
-   m->next = CBuf->spots;
-   CBuf->spots = m;
-   return(1);
+   
+   m = &a->marks[a->num_marks];
+   a->num_marks += 1;
+   
+   m->next = *mp;
+   *mp = m;
+   
+   init_mark (m, flags);
+   
+   return m;
 }
 
-int push_mark()
+/*}}}*/
+
+static void deallocate_mark (Jed_Mark_Array_Type **ap, Mark **mp) /*{{{*/
 {
-    Mark *m;
-
-   if (CBuf->mark_ptr < SPOT_ARRAY_SIZE)
+   Jed_Mark_Array_Type *a;
+   Mark *m;
+   
+   m = *mp;
+   *mp = m->next;
+   
+   a = *ap;
+   a->num_marks -= 1;
+   
+   if (a->num_marks == 0)
      {
-	m = &CBuf->mark_array[CBuf->mark_ptr++];
+	*ap = a->next;
+	SLfree ((char *)a);
      }
-   else if (NULL == (m = (Mark *) SLMALLOC(sizeof(Mark))))
-     {
-	exit_error("push_mark: malloc error!", 0);
-     }
-   /*
-    if (NULL == (m = (Mark *) SLMALLOC(sizeof(Mark))))
-      {
-	 msg_error("push_mark: malloc error!");
-	 return(0);
-      } */
-
-   m->line = CLine;
-   m->point = Point;
-   m->n = LineNum + CBuf->nup;
-   m->next = CBuf->marks;
-   m->flags = 0;
-   CBuf->marks = m;
-   return(1);
 }
 
-void goto_mark(Mark *m)
+/*}}}*/
+
+int push_spot() /*{{{*/
+{
+   (void) allocate_mark (&CBuf->spot_array, &CBuf->spots, 0);
+   return 1;
+}
+
+/*}}}*/
+
+int push_mark() /*{{{*/
+{
+   (void) allocate_mark (&CBuf->mark_array, &CBuf->marks, 0);
+   return 1;
+}
+
+/*}}}*/
+
+int goto_mark(Mark *m) /*{{{*/
 {
    Line *l;
+   int ret = -1;
    
    l = m->line;
    LineNum = m->n;
@@ -163,10 +220,14 @@ void goto_mark(Mark *m)
 	CLine = l;
 	Point = m->point;
 	LineNum -= CBuf->nup;
+	ret = 0;
      }
+   return ret;
 }
 
-int pop_mark(int *go)
+/*}}}*/
+
+int pop_mark (int *go) /*{{{*/
 {
    Mark *m;
    
@@ -180,43 +241,46 @@ int pop_mark(int *go)
 	/* touch screen since region may be highlighted */
 	if (CBuf->vis_marks == 0) touch_screen();
      }
+
+   deallocate_mark (&CBuf->mark_array, &CBuf->marks);
    
-   CBuf->marks = m->next;
-   if (m == &CBuf->mark_array[CBuf->mark_ptr - 1])
-     {
-	CBuf->mark_ptr--;
-     }
-   else SLFREE(m);
-   return(1);
+   return 1;
 }
 
-int mark_spot()
+/*}}}*/
+
+int mark_spot() /*{{{*/
 {
     push_spot();
     message("Spot Marked.");
     return(1);
 }
 
-int pop_spot()
+/*}}}*/
+
+static int pop_spot_go (int go) /*{{{*/
 {
    Mark *m;
    
    m = CBuf->spots;
    if (m == NULL) return(0);
    
-   goto_mark (m);
-   
-   CBuf->spots = m->next;
-   
-   if (m == &CBuf->spot_array[CBuf->spot_ptr - 1])
-     {
-	CBuf->spot_ptr--;
-     }
-   else SLFREE(m);
-   return(1);
+   if (go) goto_mark (m);
+
+   deallocate_mark (&CBuf->spot_array, &CBuf->spots);
+   return 1;
 }
 
-int exchange_point_mark(void)
+/*}}}*/
+
+int pop_spot () /*{{{*/
+{
+   return pop_spot_go (1);
+}
+
+/*}}}*/
+
+int exchange_point_mark(void) /*{{{*/
 {
    Line *save_line;
    int save_point;
@@ -235,13 +299,18 @@ int exchange_point_mark(void)
    return(1);
 }
 
-   
+/*}}}*/
+
+/*}}}*/
+
+/*{{{ Narrow/Widen/Region Functions */
+
  /*returns 0 if the mark is not set and gives error.  Exchanges point and mark
   * to produce valid region.  A valid region is one with mark
   * earlier in the buffer than point.  Always call this if using a region
   * which reqires point > mark.  Also, push spot first then pop at end. 
   */
-int check_region(int *push)
+int check_region(int *push) /*{{{*/
 {
    register Line *beg, *tthis = CLine;
    int pbeg;
@@ -271,8 +340,9 @@ int check_region(int *push)
    return(1);
 }
 
+/*}}}*/
 
-int widen_buffer(Buffer *b)
+static int widen_buffer_lines (Buffer *b) /*{{{*/
 {
    Narrow_Type *n;
    Buffer *save = CBuf;
@@ -286,7 +356,11 @@ int widen_buffer(Buffer *b)
    eob();
    if ((n->end != NULL)
        && (!CLine->len || ('\n' != *(CLine->data + (CLine->len - 1)))))
-     ins('\n');
+     {
+	unsigned int flags = CBuf->flags;
+	ins('\n');
+	CBuf->flags = flags;
+     }
    
    pop_spot();
    
@@ -305,27 +379,182 @@ int widen_buffer(Buffer *b)
    b->ndown -= n->ndown;
    b->narrow = n->next;
    
-   SLFREE(n);
+   SLfree ((char *)n);
    switch_to_buffer(save);
    return(1);
 }
-   
-int widen()
+
+/*}}}*/
+
+int widen_buffer (Buffer *b) /*{{{*/
 {
-   return widen_buffer(CBuf);
-}
+   unsigned int flags;
+#if JED_HAS_LINE_ATTRIBUTES
+   unsigned int line_flags;
+#endif
+   Buffer *save;
    
+   if (b->narrow == NULL) return 0;
+
+   if (b->narrow->is_region == 0)
+     return widen_buffer_lines (b);
+   
+   flags = b->flags;
+   b->flags &= ~READ_ONLY;
+   
+   save = CBuf;
+   if (b != CBuf) switch_to_buffer (b);
+   push_spot ();
+   bob ();
+   push_spot ();
+   eob ();
+   widen_buffer_lines (CBuf);
+   
+#if JED_HAS_LINE_ATTRIBUTES
+   line_flags = CLine->flags;
+   CLine->flags &= ~JED_LINE_READONLY;
+#endif
+   del ();
+#if JED_HAS_LINE_ATTRIBUTES
+   CLine->flags = line_flags;
+#endif
+   
+   pop_spot ();
+   prevline (&Number_One);
+#if JED_HAS_LINE_ATTRIBUTES
+   line_flags = CLine->flags;
+   CLine->flags &= ~JED_LINE_READONLY;
+#endif
+   del ();
+#if JED_HAS_LINE_ATTRIBUTES
+   CLine->flags = line_flags;
+#endif
+   pop_spot ();
+   if (save != CBuf) switch_to_buffer (save);
+   
+   b->flags = flags;
+   return 1;
+}
+
+/*}}}*/
+
+int narrow_to_region (void) /*{{{*/
+{
+   int pnt;
+   Line *line;
+   unsigned int flags;
+#if JED_HAS_LINE_ATTRIBUTES
+   unsigned int line_flags;
+#endif
+     
+   if (0 == check_region (&Number_One))/* spot pushed */
+     return 0;
+   
+   flags = CBuf->flags;
+   CBuf->flags &= ~READ_ONLY;
+   
+   push_spot ();
+   
+   line = CLine;
+   pnt = Point;
+   
+   pop_mark (&Number_One);
+   /* Special case if region is empty */
+   if ((CLine == line) && (pnt == Point))
+     {
+	pop_spot ();
+	pop_spot ();
+
+#if JED_HAS_LINE_ATTRIBUTES
+	line_flags = CLine->flags;
+	CLine->flags &= ~JED_LINE_READONLY;
+#endif
+	
+	newline ();
+	
+#if JED_HAS_LINE_ATTRIBUTES
+	CLine->prev->flags = line_flags;
+#endif
+	
+	push_mark ();
+
+	newline ();
+	
+#if JED_HAS_LINE_ATTRIBUTES
+	CLine->flags = line_flags;
+#endif
+	prevline (&Number_One);
+
+#if JED_HAS_LINE_ATTRIBUTES
+	CLine->flags = line_flags;
+#endif
+
+	if (narrow_to_lines ())
+	  CBuf->narrow->is_region = 1;
+     }
+   else
+     {
+#if JED_HAS_LINE_ATTRIBUTES
+	line_flags = CLine->flags;
+	CLine->flags &= ~JED_LINE_READONLY;
+#endif
+	newline ();
+#if JED_HAS_LINE_ATTRIBUTES
+	CLine->flags = line_flags;
+	CLine->prev->flags = line_flags;
+#endif
+
+	push_mark ();
+	pop_spot ();
+	
+#if JED_HAS_LINE_ATTRIBUTES
+	line_flags = CLine->flags;
+	CLine->flags &= ~JED_LINE_READONLY;
+#endif
+	newline ();
+#if JED_HAS_LINE_ATTRIBUTES
+	CLine->flags = line_flags;
+#endif
+	prevline (&Number_One);
+
+#if JED_HAS_LINE_ATTRIBUTES
+	CLine->flags = line_flags;
+#endif
+	if (narrow_to_lines ())
+	  CBuf->narrow->is_region = 1;
+	
+	pop_spot ();
+     }
+   
+   CBuf->flags = flags;
+   return 1;
+}
+
+/*}}}*/
+
+int widen () /*{{{*/
+{
+   return widen_buffer_lines (CBuf);
+}
+
+/*}}}*/
+
+int widen_region (void) /*{{{*/
+{
+   return widen_buffer (CBuf);
+}
+
+/*}}}*/
 
 /* not really a region of points but a region of lines. */
-int narrow_to_region()
+int narrow_to_lines () /*{{{*/
 {
    Line *beg;
    Narrow_Type *nt;
    
-   if (NULL == (nt = (Narrow_Type *) SLMALLOC(sizeof(Narrow_Type))))
+   if (NULL == (nt = (Narrow_Type *) SLmalloc (sizeof(Narrow_Type))))
      {
-	msg_error("Malloc Error during narrow.");
-	return(0);
+	return 0;
      }
    
    if (!check_region(&Number_One)) return(0);       /* spot pushed */
@@ -352,24 +581,40 @@ int narrow_to_region()
    nt->beg1 = CBuf->beg;
    nt->end1 = CBuf->end;
    
+   nt->is_region = 0;
    CBuf->beg = beg;
    CBuf->end = CLine;
    beg->prev = NULL;
    CLine->next = NULL;
    
+   if (CLine->len && (CLine->data[CLine->len - 1] == '\n'))
+     {
+	/* I do not think that this will affect undo. */
+	CLine->len--;
+     }
+   
    pop_spot();
    touch_windows();
    return(1);
 }
 
-int yank()
+/*}}}*/
+
+/*}}}*/
+
+/*{{{ Pastebuffer, Delete Region functions */
+
+int yank() /*{{{*/
 {
+   CHECK_READ_ONLY
     if (Paste_Buffer == NULL) return(0);
     insert_buffer(Paste_Buffer);
     return(1);
 }
 
-int copy_region_to_buffer(Buffer *b)
+/*}}}*/
+
+int copy_region_to_buffer(Buffer *b) /*{{{*/
 {
    int first_point, last_point, n, tmpm;
    Line *first, *last;
@@ -425,18 +670,21 @@ int copy_region_to_buffer(Buffer *b)
    return(1);
 }
 
-int copy_to_pastebuffer()
+/*}}}*/
+
+int copy_to_pastebuffer() /*{{{*/
 {
    /* delete paste buffer */
    if (Paste_Buffer != NULL) delete_buffer(Paste_Buffer);
-   Paste_Buffer = make_buffer();
-   strcpy(Paste_Buffer->name, " <paste>");
+   Paste_Buffer = make_buffer (" <paste>", NULL, NULL);
 
    copy_region_to_buffer(Paste_Buffer);
    return(0);
 }
 
-int delete_region (void)
+/*}}}*/
+
+int delete_region (void) /*{{{*/
 {
    int beg_point, tmpm, end_point, n;
    Line *beg, *end;
@@ -482,7 +730,9 @@ int delete_region (void)
    return(1);
 }
 
-int kill_region()
+/*}}}*/
+
+int kill_region() /*{{{*/
 {
    int tmpm = 1;
    
@@ -505,8 +755,14 @@ int kill_region()
    return(1);
 }
 
+/*}}}*/
+
+/*}}}*/
+
+/*{{{ Rectangle Functions */
+
 static char *Rect_Error = "Rectangle has 0 width.";
-int insert_rectangle()
+int insert_rectangle() /*{{{*/
 {
    int c1;
    Line *rline;
@@ -537,7 +793,9 @@ int insert_rectangle()
    return(1);
 }
 
-int open_rectangle()
+/*}}}*/
+
+int open_rectangle() /*{{{*/
 {
    int c1, n, c2, tmpm;
    Line *save_line;
@@ -570,8 +828,10 @@ int open_rectangle()
    return(1);
 }
 
+/*}}}*/
+
 /* rectangle commands */
-int copy_rectangle()
+int copy_rectangle() /*{{{*/
 {
     Line *save_line, *line, *beg;
     int c1, c2, dc, tmp, tmpm, dc_malloc;
@@ -581,8 +841,7 @@ int copy_rectangle()
     /* delete Rectangle buffer */
     if (Rectangle_Buffer != NULL) delete_buffer(Rectangle_Buffer);
 
-    Rectangle_Buffer = make_buffer();
-    strcpy(Rectangle_Buffer->name, " <rect>");
+    Rectangle_Buffer = make_buffer (" <rect>", NULL, NULL);
     c2 = calculate_column();
     save_line = CLine;
 
@@ -614,7 +873,7 @@ int copy_rectangle()
 	 /* p1 = data;
 	 p2 = data + dc;
 	 while (p1 < p2) *p1++ = ' '; */
-	 MEMSET ((char *) data, ' ', dc);
+	 SLMEMSET ((char *) data, ' ', dc);
 	 
 	 line->len = dc;
 
@@ -625,7 +884,7 @@ int copy_rectangle()
 	      p2 = CLine->data + Point;
 
 	      /* while(p1 < p2) *data++ = *p1++; */
-	      MEMCPY((char *) data, (char *) p1, (int) (p2 - p1));
+	      SLMEMCPY((char *) data, (char *) p1, (int) (p2 - p1));
 	   }
 	 if (CLine == save_line) break;
 	 CLine = CLine->next;
@@ -646,7 +905,9 @@ int copy_rectangle()
     return(0);
 }
 
-int kill_rectangle()
+/*}}}*/
+
+int kill_rectangle() /*{{{*/
 {
     Line *save_line, *line, *beg;
    int c1, c2, dc, tmp, n, tmpm, dc_malloc;
@@ -658,8 +919,7 @@ int kill_rectangle()
    /* delete Rectangle buffer */
    if (Rectangle_Buffer != NULL) delete_buffer(Rectangle_Buffer);
 
-   Rectangle_Buffer = make_buffer();
-   strcpy(Rectangle_Buffer->name, " <rect>");
+   Rectangle_Buffer = make_buffer (" <rect>", NULL, NULL);
    c2 = calculate_column();
    save_line = CLine;
 
@@ -694,7 +954,7 @@ int kill_rectangle()
 	 
 	 /* p2 = data + dc; p1 = data;
 	 while (p1 < p2) *p1++ = ' '; */
-	 MEMSET((char *) data, ' ', dc);
+	 SLMEMSET((char *) data, ' ', dc);
 	 line->len = dc;
 
 	 if (c1 == goto_column1(&c1))
@@ -704,7 +964,7 @@ int kill_rectangle()
 	      p2 = CLine->data + Point;
 	      Point = (int) (p1 - CLine->data);
 	      n = (int) (p2 - p1);
-	      MEMCPY((char *) data, (char *) p1, n);
+	      SLMEMCPY((char *) data, (char *) p1, n);
 	      deln (&n);
 	      /* while(n-- > 0)
 	        {
@@ -730,7 +990,9 @@ int kill_rectangle()
     return(0);
 }
 
-int blank_rectangle()
+/*}}}*/
+
+int blank_rectangle() /*{{{*/
 {
    int c1, n, c2, pnt, tmpm;
    Line *save_line;
@@ -773,20 +1035,29 @@ int blank_rectangle()
    return(1);
 }
 
-/*  User Marks */
+/*}}}*/
 
-typedef struct 
+/*}}}*/
+
+/*{{{ User Mark Functions */
+
+typedef struct /*{{{*/
 {
    Mark m;			       /* MUST be the first */
    Buffer *b;
 }
+
+/*}}}*/
 User_Mark_Type;
 
-static void free_user_mark (User_Mark_Type *um)
+static void free_user_mark (unsigned char type, VOID_STAR um_alias) /*{{{*/
 {
    Mark *m, *m1;
    Buffer *b;
-
+   User_Mark_Type *um;
+   
+   (void) type;
+   um = (User_Mark_Type *) um_alias;
    m1 = &um->m;
 
    /* The mark is only valid if the buffer that it was created for still
@@ -797,7 +1068,10 @@ static void free_user_mark (User_Mark_Type *um)
 	/* Unlink the mark from the chain. */
 	b = um->b;
 	m = b->user_marks;
-   
+#if JED_HAS_LINE_MARKS
+	if (m1->flags & JED_LINE_MARK) 
+	  touch_screen ();
+#endif
 	if (m == m1)	b->user_marks = m1->next;
 	else 
 	  {
@@ -806,10 +1080,12 @@ static void free_user_mark (User_Mark_Type *um)
 	  }
      }
    
-   SLFREE (um);
+   SLfree ((char *)um);
 }
 
-void free_user_marks (Buffer *b)
+/*}}}*/
+
+void free_user_marks (Buffer *b) /*{{{*/
 {
    Mark *m = b->user_marks;
    
@@ -820,7 +1096,9 @@ void free_user_marks (Buffer *b)
      }
 }
 
-static int mark_valid (Mark *m)
+/*}}}*/
+
+static int mark_valid (Mark *m) /*{{{*/
 {
    if (m->flags & MARK_INVALID)
      {
@@ -830,12 +1108,14 @@ static int mark_valid (Mark *m)
    return 1;
 }
 
-int jed_move_user_object_mark (SLuser_Object_Type *uo)
+/*}}}*/
+
+int jed_move_user_object_mark (SLang_MMT_Type *mmt) /*{{{*/
 {
    User_Mark_Type *um;
    Mark *m;
    
-   um = (User_Mark_Type *) uo->obj;
+   um = (User_Mark_Type *) SLang_object_from_mmt (mmt);
    m = &um->m;
    
    if (!mark_valid (m)) return 0;
@@ -853,87 +1133,417 @@ int jed_move_user_object_mark (SLuser_Object_Type *uo)
    return 1;
 }
 
-void move_user_mark (void)
+/*}}}*/
+
+void move_user_mark (void) /*{{{*/
 {
-   SLuser_Object_Type *uo;
+   SLang_MMT_Type *mmt;
    
-   if ((uo = SLang_pop_user_object (JED_MARK_TYPE)) == NULL) return;
-   (void) jed_move_user_object_mark (uo);
-   SLang_free_user_object (uo);
+   if (NULL == (mmt = SLang_pop_mmt (JED_MARK_TYPE)))
+     return;
+
+   (void) jed_move_user_object_mark (mmt);
+   SLang_free_mmt (mmt);
 }
 
- 
+/*}}}*/
 
-void goto_user_mark (void)
+static int x_user_mark_fun (int (*xfun)(Mark *)) /*{{{*/
 {
-   SLuser_Object_Type *uo;
+   SLang_MMT_Type *mmt;
    User_Mark_Type *um;
+   int ret = -1;
    
-   if ((uo = SLang_pop_user_object (JED_MARK_TYPE)) == NULL) return;
-   um = (User_Mark_Type *) uo->obj;
+   if (NULL == (mmt = SLang_pop_mmt (JED_MARK_TYPE)))
+     return -1;
+   um = (User_Mark_Type *) SLang_object_from_mmt (mmt);
    
    if (mark_valid (&um->m))
      {
 	if (CBuf != um->b) msg_error ("Mark not in buffer.");
 	else
-	  goto_mark (&um->m);
+	  ret = (*xfun) (&um->m);
      }
-   SLang_free_user_object (uo);
+   SLang_free_mmt (mmt);
+   return ret;
 }
 
-void create_user_mark (void)
+/*}}}*/
+
+/* It is up to calling routine to ensure that mark is in buffer. */
+static int is_mark_in_narrow (Mark *m)
+{
+   return ((m->n > CBuf->nup)
+	   && (m->n <= CBuf->nup + Max_LineNum));
+}
+
+int jed_is_user_mark_in_narrow (void)
+{
+   return x_user_mark_fun (is_mark_in_narrow);
+}
+
+void goto_user_mark (void)
+{
+   (void) x_user_mark_fun (goto_mark);
+}
+
+SLang_MMT_Type *jed_make_user_object_mark (void) /*{{{*/
 {
    User_Mark_Type *um;
-   SLuser_Object_Type *uo;
+   SLang_MMT_Type *mmt;
    Mark *m;
    
-   uo = SLang_create_user_object (JED_MARK_TYPE);
-   if (uo == NULL) return;
-   
-
-   if (NULL == (um = (User_Mark_Type *) SLMALLOC (sizeof(User_Mark_Type))))
+   if (NULL == (um = (User_Mark_Type *) SLmalloc (sizeof(User_Mark_Type))))
+     return NULL;
+     
+   if (NULL == (mmt = SLang_create_mmt (JED_MARK_TYPE, (VOID_STAR) um)))
      {
-	SLang_Error = SL_MALLOC_ERROR;
-	SLFREE (uo);
-	return;
+	SLfree ((char *) um);
+	return NULL;
      }
+
    m = &um->m;
    
-   m->line = CLine;
-   m->point = Point;
-   m->n = LineNum + CBuf->nup;
+   init_mark (m, 0);
+   
    m->next = CBuf->user_marks;
-   m->flags = 0;
    
    CBuf->user_marks = m;
    
-   um->b = CBuf;   
-   uo->obj = (long *) um;
-   SLang_push_user_object (uo);
+   um->b = CBuf;
+
+   return mmt;
 }
 
-   
-   
-char *user_mark_buffer (void)
+/*}}}*/
+
+
+void create_user_mark (void) /*{{{*/
 {
-   SLuser_Object_Type *uo;
+   SLang_MMT_Type *mmt;
+   
+   if (NULL != (mmt = jed_make_user_object_mark ()))
+     {
+	if (-1 == SLang_push_mmt (mmt))
+	  SLang_free_mmt (mmt);
+     }
+}
+
+/*}}}*/
+
+char *user_mark_buffer (void) /*{{{*/
+{
+   SLang_MMT_Type *mmt;
    User_Mark_Type *um;
    char *s = "";
    
-   if (NULL == (uo = SLang_pop_user_object (JED_MARK_TYPE))) return s;
-   um = (User_Mark_Type *) uo->obj;
+   if (NULL == (mmt = SLang_pop_mmt (JED_MARK_TYPE)))
+     return s;
+
+   um = (User_Mark_Type *) SLang_object_from_mmt (mmt);
    
    if (mark_valid (&um->m))
      {
 	s = um->b->name;
      }
    
-   SLang_free_user_object (uo);
+   SLang_free_mmt (mmt);
    return s;
 }
 
-   
-int register_jed_classes (void)
+/*}}}*/
+
+static int
+user_mark_bin_op_result (int op, unsigned char a, unsigned char b,
+			 unsigned char *c)
 {
-   return SLang_register_class (JED_MARK_TYPE, (VOID *) free_user_mark, NULL);
+   (void) a; (void) b;
+   switch (op)
+     {
+      default: 
+	return 0;
+	
+      case SLANG_GT:
+      case SLANG_GE:
+      case SLANG_LT:
+      case SLANG_LE:
+      case SLANG_EQ:
+      case SLANG_NE:
+	*c = SLANG_INT_TYPE;
+	break;
+     }
+   return 1;
 }
+
+
+static int
+user_mark_bin_op (int op,
+		  unsigned char a_type, VOID_STAR ap, unsigned int na,
+		  unsigned char b_type, VOID_STAR bp, unsigned int nb,
+		  VOID_STAR cp)
+{
+   int *ic;
+   unsigned int n, n_max;
+   unsigned int da, db;
+   SLang_MMT_Type **a, **b;
+
+   (void) a_type;
+   (void) b_type;
+
+   if (na == 1) da = 0; else da = 1;
+   if (nb == 1) db = 0; else db = 1;
+
+   if (na > nb) n_max = na; else n_max = nb;
+
+   a = (SLang_MMT_Type **) ap;
+   b = (SLang_MMT_Type **) bp;
+
+   ic = (int *) cp;
+
+   for (n = 0; n < n_max; n++)
+     {
+	User_Mark_Type *ua, *ub;
+	Buffer *ba, *bb;
+	int pa, pb;
+	unsigned int la, lb;
+
+	ua = ub = NULL;
+	ba = bb = NULL;
+	pa = pb = 0;
+	la = lb = 0;
+	
+	if ((*a != NULL)
+	    && (NULL != (ua = (User_Mark_Type *) SLang_object_from_mmt (*a))))
+	  {
+	     la = ua->m.n;
+	     pa = ua->m.point;
+	     ba = ua->b;
+	  }
+
+	if ((*b != NULL)
+	    && (NULL != (ub = (User_Mark_Type *) SLang_object_from_mmt (*b))))
+	  {
+	     lb = ub->m.n;
+	     pb = ub->m.point;
+	     bb = ub->b;
+	  }
+
+	if ((ba == NULL) || (bb == NULL))
+	  {
+	     ic[n] = 0;
+	     a += da;
+	     b += db;
+	     continue;
+	  }
+	
+	switch (op)
+	  {
+	   case SLANG_GT:
+	     ic[n] = ((la > lb)
+		      || ((la == lb) && (pa > pb)));
+	     break;
+	     
+	   case SLANG_GE:
+	     ic[n] = ((la > lb)
+		      || ((la == lb) && (pa >= pb)));
+	     break;
+	     
+	   case SLANG_LT:
+	     ic[n] = ((la < lb)
+		      || ((la == lb) && (pa < pb)));
+	     break;
+
+	   case SLANG_LE:
+	     ic[n] = ((la < lb)
+		      || ((la == lb) && (pa <= pb)));
+	     break;
+	     
+	   case SLANG_EQ:
+	     ic[n] = ((ba == bb) && (la == lb) && (pa == pb));
+	     break;
+
+	   case SLANG_NE:
+	     ic[n] = ((ba != bb) || (la != lb) || (pa != pb));
+	     break;
+	     
+	   default:
+	     return 0;
+	  }
+
+	a += da;
+	b += db;
+     }
+
+   return 1;
+}
+
+
+
+
+/*}}}*/
+
+#if JED_HAS_LINE_MARKS
+void jed_create_line_mark (int *color) /*{{{*/
+{
+   SLang_MMT_Type *mmt;
+   User_Mark_Type *um;
+   
+   if (NULL == (mmt = jed_make_user_object_mark ()))
+     return;
+
+   um = (User_Mark_Type *) SLang_object_from_mmt (mmt);
+   um->m.flags |= JED_LINE_MARK | (*color & MARK_COLOR_MASK);
+   if (-1 == SLang_push_mmt (mmt))
+     SLang_free_mmt (mmt);
+}
+
+/*}}}*/
+#endif
+   
+
+int register_jed_classes (void) /*{{{*/
+{
+   SLang_Class_Type *cl;
+   
+   cl = SLclass_allocate_class ("Mark_Type");
+   if (cl == NULL) return -1;
+   (void) SLclass_set_destroy_function (cl, free_user_mark);
+   
+   if (-1 == SLclass_register_class (cl, JED_MARK_TYPE, sizeof (User_Mark_Type), SLANG_CLASS_TYPE_MMT))
+     return -1;
+   
+   if (-1 == SLclass_add_binary_op (JED_MARK_TYPE, JED_MARK_TYPE, user_mark_bin_op, user_mark_bin_op_result))
+     return -1;
+   
+   return 0;
+}
+
+/*}}}*/
+
+void jed_widen_whole_buffer (Buffer *b) /*{{{*/
+{
+   while (b->narrow != NULL) widen_buffer (b);
+}
+
+/*}}}*/
+
+#if JED_HAS_SAVE_NARROW
+static void restore_saved_narrow (void) /*{{{*/
+{
+   Mark *beg, *end;
+   
+   Jed_Save_Narrow_Type *save_narrow;
+   
+   if (NULL == (save_narrow = CBuf->save_narrow))
+     return;
+   
+   push_spot ();
+   /* remove current restriction */
+   jed_widen_whole_buffer (CBuf);
+   
+   beg = save_narrow->beg;
+   end = save_narrow->end;
+   
+   while (beg != NULL)
+     {
+	goto_mark (beg);
+	push_mark ();
+	goto_mark (end);
+	if (end->flags & NARROW_REGION_MARK)
+	  narrow_to_region ();
+	else narrow_to_lines ();
+	
+	beg = beg->next;
+	end = end->next;
+     }
+   pop_spot ();
+}
+
+/*}}}*/
+static void free_mark_chain (Mark *m) /*{{{*/
+{
+   Mark *next;
+	
+   while (m != NULL)
+     {
+	next = m->next;
+	SLfree ((char *)m);
+	m = next;
+     }
+}
+
+/*}}}*/
+void jed_free_saved_narrow (void) /*{{{*/
+{
+   Jed_Save_Narrow_Type *save_narrow;
+   
+   save_narrow = CBuf->save_narrow;
+   if (save_narrow == NULL) return;
+   
+   CBuf->save_narrow = save_narrow->next;
+   
+   free_mark_chain (save_narrow->beg);
+   free_mark_chain (save_narrow->end);
+   SLfree ((char *)save_narrow);
+}
+
+/*}}}*/
+void jed_push_narrow (void) /*{{{*/
+{
+   Jed_Save_Narrow_Type *save_narrow;
+   
+   if (NULL == (save_narrow = (Jed_Save_Narrow_Type *) SLmalloc (sizeof (Jed_Save_Narrow_Type))))
+     {
+	exit_error ("push_narrow: malloc error.", 0);
+     }
+   save_narrow->beg = save_narrow->end = NULL;
+   save_narrow->next = CBuf->save_narrow;
+   CBuf->save_narrow = save_narrow;
+   
+   push_spot ();
+   while (CBuf->narrow != NULL)
+     {
+	Mark *m;
+	
+	bob ();
+	m = create_mark (0);
+	m->next = save_narrow->beg;
+	save_narrow->beg = m;
+	
+	eob ();
+	m = create_mark (CBuf->narrow->is_region ? NARROW_REGION_MARK : 0);
+	m->next = save_narrow->end;
+	save_narrow->end = m;
+	
+	widen_buffer (CBuf);
+     }
+   
+   restore_saved_narrow ();
+   
+   pop_spot ();
+}
+
+/*}}}*/
+void jed_pop_narrow (void) /*{{{*/
+{
+   restore_saved_narrow ();
+   jed_free_saved_narrow ();
+}
+
+/*}}}*/
+#endif
+
+int jed_count_narrows (void) /*{{{*/
+{
+   int n = 0;
+   Narrow_Type *nt = CBuf->narrow;
+   
+   while (nt != NULL)
+     {
+	n++;
+	nt = nt->next;
+     }
+   return n;
+}
+
+/*}}}*/
