@@ -14,7 +14,7 @@
 #include "jdmacros.h"
 
 #include <string.h>
-#ifdef pc_system
+#ifdef IBMPC_SYSTEM
 #include <dos.h>
 #endif
 
@@ -42,12 +42,8 @@
 
 /*{{{ Jed Version */
 
-#ifdef FLOAT_TYPE
-# define JED_VERSION "0.F98.4"
-#else
-# define JED_VERSION "0.98.4"
-#endif
-int Jed_Version_Number = 9804;
+# define JED_VERSION "0.98.7"
+int Jed_Version_Number = 9807;
 
 /*}}}*/
 
@@ -179,13 +175,13 @@ static int output(unsigned char *line, int len, int row, register int max_col, /
    int done = 0;
    unsigned short dollar = Jed_Dollar & 0xFF;
    
-#if !defined(pc_system) && !defined(__os2__)
+#if !defined(IBMPC_SYSTEM) && !defined(__os2__)
    dollar |= 0x20;		       /* prevent control character */
    if (*tt_Use_Ansi_Colors && Term_Supports_Color)
      {
 #endif
 	dollar |= JDOLLAR_COLOR << 8;
-#if !defined(pc_system) && !defined(__os2__)
+#if !defined(IBMPC_SYSTEM) && !defined(__os2__)
      }
 #endif
    
@@ -327,7 +323,7 @@ static int output(unsigned char *line, int len, int row, register int max_col, /
 	  }
      }
    
-#if !defined(pc_system) && !defined(__os2__)
+#if !defined(IBMPC_SYSTEM) && !defined(__os2__)
    if (*tt_Use_Ansi_Colors && Term_Supports_Color)
      {
 #endif
@@ -338,7 +334,7 @@ static int output(unsigned char *line, int len, int row, register int max_col, /
 	     unsigned short *ppp = out;
 	     while (ppp < outp) *ppp++ |= Message_Color;
 	  }
-#if !defined(pc_system) && !defined(__os2__)
+#if !defined(IBMPC_SYSTEM) && !defined(__os2__)
      }
 #endif
    
@@ -378,7 +374,7 @@ static int output(unsigned char *line, int len, int row, register int max_col, /
    
    sp = outp;
    
-#if defined(msdos) && !defined(__WATCOMC__) && !defined(__WIN32__)
+#if defined(IBMPC_USE_ASM)
    asm mov bx, di
      asm mov cx, word ptr spmax
      asm sub cx, word ptr outp
@@ -433,13 +429,7 @@ static int output(unsigned char *line, int len, int row, register int max_col, /
    
    if (visible)
      {
-#if defined(msdos) && !defined(MSWINDOWS)
-	/* 2nd and last parameters not used in msdos smartputs so they do
-	   not need to be valid */
-	tt_smart_puts(out, out, *tt_Screen_Cols, row - 1);
-#else
 	tt_smart_puts(out, JScreen[row-1].old, *tt_Screen_Cols, row - 1);
-#endif
 	JScreen[row-1].n = visible;
 	Point_Cursor_Flag = 1;
      }
@@ -1899,33 +1889,35 @@ void set_status_format(char *f, int *local) /*{{{*/
 
 static void make_status_line(int col_flag) /*{{{*/
 {
-   unsigned short mrk, flag, spot, star, ubit = '-';
+   unsigned short mrk, flag, spot, star0, star1, ubit = '-';
    register unsigned short *s1, *s, *smax;
    
    if (JWindow->top == *tt_Screen_Rows) return;   /* minibuffer ? */
    s = JScreen[JWindow->rows + JWindow->top - 1].neew;
    smax = s + JWindow->width;
-   mrk = flag = spot = star = '-';
+   mrk = flag = spot = star0 = star1 = '-';
    if (CBuf->marks != NULL) mrk = 'm';
    if (CBuf->flags & FILE_MODIFIED) flag = 'd';
    if (CBuf->spots != NULL) spot = 's';
-   if (CBuf->flags & BUFFER_TRASHED) star = '*';
+   if (CBuf->flags & BUFFER_TRASHED) star0 = star1 = '*';
    if ((CBuf->flags & READ_ONLY)
 #if JED_HAS_LINE_ATTRIBUTES
        || (CLine->flags & JED_LINE_READONLY)
 #endif
        )
-     star = '%';
-
+     star0 = star1 = '%';
+#if JED_HAS_SUBPROCESSES
+   if (CBuf->subprocess) star1 = 'S';
+#endif
    if (CBuf->flags & UNDO_ENABLED) ubit = '+';
    
    s1 = s;
    if (JWindow->column != 1) *s1++ = '<'; else *s1++ = '-';
-   *s1++ = star; *s1++ = star; *s1++ = mrk; *s1++ = flag;
+   *s1++ = star0; *s1++ = star1; *s1++ = mrk; *s1++ = flag;
    *s1++ = spot;
    
    if (CBuf->flags & BINARY_FILE) *s1++ = 'B';
-#ifdef pc_system
+#ifdef IBMPC_SYSTEM
    else if ((CBuf->flags & ADD_CR_ON_WRITE_FLAG) == 0) *s1++ = 'L';
 #else
 #ifdef __unix__
@@ -2092,7 +2084,7 @@ static int update_1(Line *top, int force) /*{{{*/
 #endif
 	     
 	      /* the next line is really optional */
-#ifndef pc_system
+#if 0
 	     if (!force && (Exit_From_MiniBuffer ||
 			    screen_input_pending (0))) break;
 #endif
@@ -2228,7 +2220,7 @@ void do_dialog(char *b) /*{{{*/
    
    if ((b == Error_Buffer) || (SLKeyBoard_Quit))
      {
-	beep();
+	jed_beep();
 	flush_input();
      }
    
@@ -2403,7 +2395,7 @@ void update(Line *line, int force, int flag) /*{{{*/
 	else
 	  {
 	     last_time = Status_This_Time;
-	     if (SLang_run_hooks ("update_timer_hook", NULL, NULL))
+	     if (SLang_run_hooks ("update_timer_hook", 0))
 	       flag = 0;
 	  }
      }
@@ -2522,24 +2514,24 @@ static int Display_Initialized;
 
 void reset_display (void) /*{{{*/
 {
-   int i;
-   unsigned short *p;
+   unsigned int i, imax;
    
    if (Display_Initialized == 0) return;
    if (Batch) return;
+   
    Display_Initialized = 0;
 
 #if defined(VMS) || defined(REAL_UNIX_SYSTEM)
-   SLang_run_hooks ("reset_display_hook", NULL, NULL);
+   (void) SLang_run_hooks ("reset_display_hook", 0);
 #endif
-
+   
+   imax = *tt_Screen_Rows;
    tt_reset_video ();
-   /* tt_del_eol ();
-    flush_output (); */
-   for (i = 0; i < *tt_Screen_Rows; i++)
+   
+   for (i = 0; i < imax; i++)
      {
-	p = JScreen[i].old;  if (p != NULL) SLFREE(p); JScreen[i].old = NULL;
-	p = JScreen[i].neew;  if (p != NULL) SLFREE(p); JScreen[i].neew = NULL;
+	SLfree ((char *) JScreen[i].old); JScreen[i].old = NULL;
+	SLfree ((char *) JScreen[i].neew); JScreen[i].neew = NULL;
      }
 }
 
@@ -2570,7 +2562,8 @@ void init_display (int g) /*{{{*/
 	*tt_Screen_Rows = r;
      }
    
-   tt_init_video ();
+   if (-1 == tt_init_video ())
+     exit_error ("init_display: error initializing display", 0);
    
    c = *tt_Screen_Cols;
    r = *tt_Screen_Rows;
@@ -2579,8 +2572,8 @@ void init_display (int g) /*{{{*/
      {
 	/* these are bigger to handle overflow in output routine in case
 	 * special char occurs at end as well as allowing code as in blank_line*/
-	if ((NULL == (old = (unsigned short *) SLCALLOC(c + 3, sizeof(short))))
-	    || (NULL == (neew = (unsigned short *) SLCALLOC(c + 3, sizeof(short)))))
+	if ((NULL == (old = (unsigned short *) SLcalloc (c + 3, sizeof(short))))
+	    || (NULL == (neew = (unsigned short *) SLcalloc (c + 3, sizeof(short)))))
 	  {
 	     exit_error("init_display(): malloc error.", 0);
 	  }
@@ -2595,7 +2588,7 @@ void init_display (int g) /*{{{*/
      }
 
 #if defined(VMS) || defined(REAL_UNIX_SYSTEM)
-   SLang_run_hooks ("init_display_hook", NULL, NULL);
+   (void) SLang_run_hooks ("init_display_hook", 0);
 #endif
 
    Display_Initialized = 1;
@@ -2611,7 +2604,7 @@ void redraw_screen(int force) /*{{{*/
    
    
    if (Batch) return;
-#ifndef msdos
+#ifndef IBMPC_SYSTEM
    /* send_string_to_term("\033[?6h"); */  /* relative origin mode */
    tt_reset_scroll_region();
    tt_end_insert();
@@ -2802,7 +2795,6 @@ void exit_error(char *str, int severity) /*{{{*/
 	     if (JWindow != NULL) fprintf(stderr, "JWindow: %p, top: %d, rows: %d, buffer: %p\n",
 					  JWindow, JWindow->top, JWindow->rows, JWindow->buffer);
 	  }
-	auto_save_all();
      }
    if (severity)
      {
