@@ -1,10 +1,13 @@
+/* -*- mode: C; mode: fold; -*- */
 /*
  *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
  *  All Rights Reserved.
  */
+#include "config.h"
+#include "jed-feat.h"
+
 #include <stdio.h>
 #include <string.h>
-#include "config.h"
 #include "sysdep.h"
 
 #include <ssdef.h>
@@ -24,6 +27,10 @@
 #include <unixlib.h> 
 #include <unixio.h>
 #endif
+
+#if (__VMS_VER >= 70000000)
+# include <unistd.h>
+#endif
 /* #include <libdef.h>  */
 
 typedef struct {                /* I/O status block     */
@@ -48,17 +55,20 @@ int Abort_Char = 7;		       /* scan code for G (control) */
 short This_Term;
 
 
-typedef struct
+typedef struct /*{{{*/
 {
    short buflen;
    short item_code;
    int *buf_addr;
    int *len_addr;
-} item_list_3;
-   
+} 
+/*}}}*/
+item_list_3;
+
+
 static int TTY_Inited;
 
-void vms_exit_handler(int not_used)
+void vms_exit_handler(int not_used) /*{{{*/
 {
    if (TTY_Inited == 0) exit(0);
 
@@ -68,6 +78,8 @@ void vms_exit_handler(int not_used)
    exit(1);
 }
 
+/*}}}*/
+
 /*
  *      Exit Handler Control Block
  */
@@ -88,18 +100,20 @@ exit_block =
       0
   };
 
-void vms_cancel_exithandler()
+void vms_cancel_exithandler() /*{{{*/
 {
    sys$canexh(exit_block);
 }
 
+/*}}}*/
+
 #undef USING_INPUT_BUFFER
 #undef DONE_WITH_INPUT_BUFFER
 
 
 int vms_input_buffer;
 
-static struct vms_ast_iosb
+static struct vms_ast_iosb /*{{{*/
 {
     short status;
     short offset;
@@ -107,6 +121,8 @@ static struct vms_ast_iosb
     short term;
 } vms_ast_iosb;
 
+/*}}}*/
+
 extern void vms_que_key_ast();
 static int Ast_Fired_Event_Flag;
 static int Timer_Event_Flag;
@@ -121,7 +137,7 @@ static int Using_Keyboard_Buffer_Event_Flag;
 #define DONE_WITH_INPUT_BUFFER\
    sys$setast(1);
 
-static int getkey_ast(int not_used)
+static int getkey_ast(int not_used) /*{{{*/
 {
    unsigned int c = 1000;
    
@@ -151,7 +167,9 @@ static int getkey_ast(int not_used)
    return (1);
 }
 
-void vms_que_key_ast()
+/*}}}*/
+
+void vms_que_key_ast() /*{{{*/
 {
    static int trmmsk [2] = { 0, 0 };
    int status;
@@ -163,10 +181,12 @@ void vms_que_key_ast()
 		     &vms_input_buffer, 1, 0, trmmsk, 0, 0);
 }
 
+/*}}}*/
+
 static char TTY_Name[8];
 static int This_Process_Pid;
 
-void init_tty()
+int init_tty() /*{{{*/
 {
    Iosb_Type iostatus;
    int tmp, name_len, status, lastppid, ppid;
@@ -204,7 +224,7 @@ void init_tty()
 	if (status != SS$_NORMAL) 
 	  {
 	     fprintf(stderr, "PID: %X, status: %X\n", This_Process_Pid, status);
-	     exit(1);
+	     return -1;
 	  }
 
 	if (lastppid == ppid) break;
@@ -214,11 +234,11 @@ void init_tty()
 	itmlst[0].buf_addr =  &ppid;
      }
    
-   if (Batch) return;
+   if (Batch) return 0;
    if (X_Init_Term_Hook != NULL)
      {
 	(void) (*X_Init_Term_Hook) ();
-	return;
+	return 0;
      }
    
    term.dsc$w_length = name_len;
@@ -227,7 +247,7 @@ void init_tty()
      {
 	fprintf(stderr,"Unable to assign input channel\n");
 	fprintf(stderr,"PID: %X, DEV %s, status: %d\n", This_Process_Pid, TTY_Name, status);
-	exit(0);
+	return -1;
      }
 
    if (NULL == exit_block.exit_routine)
@@ -290,13 +310,18 @@ void init_tty()
    /* Enable the Application Keypad */
    (*tt_write_string) ("\033=\033[?1l");   /* application keys/cursor keys */
    vms_que_key_ast();   /* set up the key ast */
+   return 0;
 }
 
-void sys_flush(int fd)
+/*}}}*/
+
+void sys_flush(int fd) /*{{{*/
 {
 }
 
-static void cancel_ast (void)
+/*}}}*/
+
+static void cancel_ast (void) /*{{{*/
 {
    if (TTY_Inited == 0) return;
    
@@ -318,8 +343,10 @@ static void cancel_ast (void)
    Waiting_For_Ast = 0;
 }
 
+/*}}}*/
+
 static int keypad_state = 0;
-void reset_tty()
+void reset_tty() /*{{{*/
 {
    Iosb_Type iostatus;
    if (Batch) return;
@@ -348,7 +375,9 @@ void reset_tty()
    else SLtt_write_string("\033>");
 }
 
-unsigned char sys_getkey()
+/*}}}*/
+
+unsigned char sys_getkey() /*{{{*/
 {
    unsigned char c;
    int tsecs;
@@ -367,7 +396,7 @@ unsigned char sys_getkey()
    if (Input_Buffer_Len) return(my_getkey());
    tsecs = 450;   /* 45 seconds */
    
-   while (!sys_input_pending(&tsecs))
+   while (!sys_input_pending(&tsecs, 0))
      {
 	/* update status line incase user is displaying time */
 	if (Display_Time)
@@ -380,8 +409,10 @@ unsigned char sys_getkey()
    return(c);
 }
 
+/*}}}*/
+
 /* waits *secs tenth of seconds for input */
-static int sys_input_pending(int *secs)
+static int sys_input_pending(int *secs, int unused) /*{{{*/
 {
    unsigned long daytim[2];
    
@@ -425,8 +456,28 @@ static int sys_input_pending(int *secs)
    return (Input_Buffer_Len);
 }
 
+/*}}}*/
+
+/* This is completely untested!! */
+void sys_pause (int ms) /*{{{*/
+{
+   unsigned long daytim[2];
+
+   /* takes a quad word time.  If negative, use a relative time. */
+   daytim[1] = 0xFFFFFFFF;
+   daytim[0] = -(ms * 10 * 1000);   /* 10 * 1000 is a milli sec */
+	
+	
+   sys$setimr(Timer_Event_Flag, daytim, 0, 1);
+		   
+   /* cancel the timer */
+   sys$cantim(1, 3);   /* 3 is user mode */
+}
+
+/*}}}*/
+
 /*  This is to get the size of the terminal  */
-int get_term_dimensions(int *cols, int *rows)
+int get_term_dimensions(int *cols, int *rows) /*{{{*/
 {
    int status, junk;
    Iosb_Type iostatus;
@@ -477,19 +528,25 @@ int get_term_dimensions(int *cols, int *rows)
    return 0;
 }
 
+/*}}}*/
+
 /* returns 0 on failure, 1 on sucess */
-int sys_delete_file(char *filename)
+int sys_delete_file(char *filename) /*{{{*/
 {
     return (1 + delete(filename));   /* 0: sucess; -1 failure */
 }
 
-int sys_rename(char *from, char *to)
+/*}}}*/
+
+int sys_rename(char *from, char *to) /*{{{*/
 {
     return(-1);
 }
 
+/*}}}*/
+
 /* This routine converts unix type names to vms names */
-int locate(char ch, char *string)
+int locate(char ch, char *string) /*{{{*/
 {
     int i;
     char c;
@@ -499,7 +556,9 @@ int locate(char ch, char *string)
     if (c == ch) return(i); else return (0);
 }
 
-char *unix2vms(char *file)
+/*}}}*/
+
+char *unix2vms(char *file) /*{{{*/
 {
     int i,device,j,first,last;
     static char vms_name[80];
@@ -583,7 +642,9 @@ char *unix2vms(char *file)
     return (vms_name);
 }
 
-void define_logical_name (char *varname, char *string)
+/*}}}*/
+
+void define_logical_name (char *varname, char *string) /*{{{*/
 {
    struct dsc$descriptor_s strdsc, envdsc, lnmdsc;
    
@@ -603,7 +664,9 @@ void define_logical_name (char *varname, char *string)
    lib$set_logical (&envdsc, &strdsc, &lnmdsc, 0, 0);
 }
 
-void delete_logical_name (char *varname)
+/*}}}*/
+
+void delete_logical_name (char *varname) /*{{{*/
 {
    struct dsc$descriptor_s envdsc, lnmdsc;
    
@@ -619,7 +682,9 @@ void delete_logical_name (char *varname)
    lib$delete_logical (&envdsc, &lnmdsc);
 }
 
-int do_attach_cmd()
+/*}}}*/
+
+int do_attach_cmd() /*{{{*/
 {
    unsigned long pid;
    char *pidstr;
@@ -636,14 +701,15 @@ int do_attach_cmd()
    else return(0);
 }
 
+/*}}}*/
+
 unsigned long SHELL_PID = 0;
 
 /* here we try to attach to the parent otherwise just spawn a new one */
-void sys_suspend()
+void sys_suspend() /*{{{*/
 {
    unsigned long parent_pid;
    unsigned long status = 0;
-   char str[80];
 
    cancel_ast ();
    parent_pid = getppid();
@@ -668,8 +734,7 @@ void sys_suspend()
 	/* if we attach back, status may come back unchanged */
 	if (!(status & 1))  /* Thanks to Hunter Goatley for this suggestion */
 	  {              
-	     sprintf(str,"Unable to spawn subprocess. Error = X%X (%d)", status, status);
-	     msg_error(str);
+	     jed_verror ("Unable to spawn subprocess. Error = X%X (%d)", status, status);
 	     return;
 	  }
      }
@@ -681,13 +746,15 @@ void sys_suspend()
      } */
 }
 
+/*}}}*/
+
 /* returns 0 on success, -1 on syntax error
    -2 on bad dir, -3 on bad device, 1 if something else */
-int vms_parse_file(char *old)
+int vms_parse_file(char *old) /*{{{*/
 {
    struct FAB fab = cc$rms_fab;
    struct NAM nam = cc$rms_nam;
-   char neew[256];
+   char neew[JED_MAX_PATH_LEN];
    int status;
 
    fab.fab$l_fna = old;
@@ -696,7 +763,7 @@ int vms_parse_file(char *old)
    fab.fab$w_ifi = 0;
    fab.fab$l_nam = &nam;
    nam.nam$l_esa = neew;
-   nam.nam$b_ess = 255;
+   nam.nam$b_ess = JED_MAX_PATH_LEN - 1;
    nam.nam$b_nop = NAM$V_SYNCHK;   /* syntax only */
    nam.nam$l_rlf = 0;
 
@@ -723,7 +790,9 @@ int vms_parse_file(char *old)
      }
 }
 
-char *expand_filename(char *file)
+/*}}}*/
+
+char *expand_filename(char *file) /*{{{*/
 {
    char *p, *p1, *dir;
    static char work[300];
@@ -806,10 +875,12 @@ char *expand_filename(char *file)
 
 }
 
-int vms_expand_filename(char *file,char *expanded_file)
+/*}}}*/
+
+int vms_expand_filename(char *file,char *expanded_file) /*{{{*/
 {
     static int context = 0;
-    static char inputname[256] = "";
+    static char inputname[JED_MAX_PATH_LEN] = "";
     $DESCRIPTOR(file_desc,inputname);
     $DESCRIPTOR(default_dsc,"SYS$DISK:[]*.*;");
     static struct dsc$descriptor_s  result =
@@ -829,7 +900,7 @@ int vms_expand_filename(char *file,char *expanded_file)
     if (RMS$_NORMAL == lib$find_file(&file_desc,&result,&context,
 	           		     &default_dsc,0,0,&Number_Zero))
       {
-	  MEMCPY(expanded_file, result.dsc$a_pointer, result.dsc$w_length);
+	  SLMEMCPY(expanded_file, result.dsc$a_pointer, result.dsc$w_length);
 	  expanded_file[result.dsc$w_length] = '\0';
           return (1);
       }
@@ -840,15 +911,17 @@ int vms_expand_filename(char *file,char *expanded_file)
       }
 }
 
+/*}}}*/
+
 static int context = 0;
 
-static char inputname[256] = "";
+static char inputname[JED_MAX_PATH_LEN] = "";
 $DESCRIPTOR(file_desc,inputname);
 $DESCRIPTOR(default_dsc,"SYS$DISK:[]*.*;");
 
 static char *VMS_Star;
 
-int sys_findnext(char *file)
+int sys_findnext(char *file) /*{{{*/
 {
    char *f;
    static struct dsc$descriptor_s  result = {0, DSC$K_DTYPE_T, DSC$K_CLASS_D, NULL};
@@ -856,7 +929,7 @@ int sys_findnext(char *file)
    if (RMS$_NORMAL == lib$find_file(&file_desc,&result,&context,
 				    &default_dsc,0,0,&Number_Zero))
      {
-	MEMCPY(file, result.dsc$a_pointer, result.dsc$w_length);
+	SLMEMCPY(file, result.dsc$a_pointer, result.dsc$w_length);
 	file[result.dsc$w_length] = 0;
 	f = &file[result.dsc$w_length];
 	if (*VMS_Star)
@@ -870,7 +943,9 @@ int sys_findnext(char *file)
    else return(0);
 }
 
-int sys_findfirst(char *thefile)
+/*}}}*/
+
+int sys_findfirst(char *thefile) /*{{{*/
 {
    char *f, *f1, *s2 = "*.*";
    char *file;
@@ -913,10 +988,12 @@ int sys_findfirst(char *thefile)
    return sys_findnext(thefile);
 }
 
+/*}}}*/
+
 #include <stat.h>
 
 /* returns 0 if file does not exist, 1 if it is not a dir, 2 if it is */
-int sys_chmod(char *file, int what, int *mode, short *uid, short *gid)
+int sys_chmod(char *file, int what, int *mode, short *uid, short *gid) /*{{{*/
 {
    struct stat buf;
    int m;
@@ -938,22 +1015,9 @@ int sys_chmod(char *file, int what, int *mode, short *uid, short *gid)
    return(1);
 }
 
-#if 0
-   switch(vms_parse_file(file))
-     {
-	case 0: break;
-	case -1:		       /* Filename syntax error. */
-	case -2:   return(-2);	       /* Directory does not exist! */
-	case 1:
-	case -3:   return(-3);	       /* Bad device name. */
-      default: return(-3);
-     }
-   return(1);
+/*}}}*/
 
-
-#endif
-
-unsigned long sys_file_mod_time(char *file)
+unsigned long sys_file_mod_time(char *file) /*{{{*/
 {
    struct stat buf;
 
@@ -961,10 +1025,16 @@ unsigned long sys_file_mod_time(char *file)
    return((unsigned long) buf.st_mtime);
 }
 
+/*}}}*/
 
-int rmdir (char *d)
+#if (__VMS_VER < 70000000)
+int rmdir (char *d) /*{{{*/
 {
    return(-1);
 }
 
+/*}}}*/
+
+#endif
+
 
