@@ -5,10 +5,11 @@
 #ifndef __JED_WINDOW_H_
 #define __JED_WINDOW_H_
 #include "buffer.h"
+
 typedef struct Window_Type
   {
       int top;                  /* top row of window */
-      int rows;                 /* number of rows */
+      int rows;                 /* number of rows (does not include status line) */
       int width;
       int column;               /* starts at 1 */
       Mark beg;
@@ -37,5 +38,10 @@ extern void window_buffer(Buffer *);
 extern void change_screen_size(int, int);
 extern int buffer_visible(Buffer *);
 extern int Top_Window_Row;
+
+#if JED_HAS_SUBPROCESSES
+extern void move_window_marks (int);
+#endif
+
 #endif
 
